{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFindReplace.PAS, released on 2002-05-26.

The Initial Developer of the Original Code is Peter Thrnqvist [peter3 at sourceforge dot net]
Portions created by Peter Thrnqvist are Copyright (C) 2002 Peter Thrnqvist.
All Rights Reserved.

Contributor(s):
  Olivier Sannier

You may retrieve the latest version of this file at the Project JEDI's JVCL home page,
located at http://jvcl.sourceforge.net

Description:
  Wrapper for the TFind / TReplace dialogs and a stand-alone full
  text search engine with support for all available dialog options:
  Search up/down, whole word only, case sensitive, replace all etc.

Known Issues:
-----------------------------------------------------------------------------}
// $Id: JvFindReplace.pas,v 1.26 2005/01/17 20:57:40 remkobonte Exp $

unit JvFindReplace;

{$I jvcl.inc}

interface

uses
  SysUtils, Classes, Windows, Messages, Controls, Dialogs, StdCtrls,
  JvComponent;

type
  TJvReplaceProgressEvent = procedure(Sender: TObject; Position: Integer;
    var Terminate: Boolean) of object;
  TJvReplaceAllEvent = procedure(Sender: TObject; ReplaceCount: Integer) of object;
 
  TJvFindReplace = class(TJvComponent)
  private
    FOnFind: TNotifyEvent;
    FOnReplace: TNotifyEvent;
    FOnReplacingAll: TNotifyEvent;
    FOnReplacedAll: TJvReplaceAllEvent;
    FOnShow: TNotifyEvent;
    FOnClose: TNotifyEvent;
    FOnNotFound: TNotifyEvent;
    FOnProgress: TJvReplaceProgressEvent;
    FEditControl: TCustomEdit;
    FOwner: TComponent;
    FFindDialog: TFindDialog;
    FReplaceDialog: TReplaceDialog;
    FOptions: TFindOptions;
    FPosition: TPoint;
///simes    FLeft: Integer;
///simes    FTop: Integer;
    FFast: Boolean;
    FHelpContext: THelpContext;
    FShowDialogs: Boolean;
///simes    FKeepText: Boolean;
    FFindText: string;
    FReplaceText: string;
    FNumberReplaced: Integer; // only used by Replace All
    procedure SetPosition(Value: TPoint);
    procedure SetDialogTop(Value: Integer);
    procedure SetDialogLeft(Value: Integer);
    procedure SetOptions(Value: TFindOptions);
    procedure SetEditControl(Value: TCustomEdit);
    procedure SetHelpContext(Value: THelpContext);
    procedure SetFindText(const Value: string);
    procedure SetReplaceText(const Value: string);
    procedure SetShowDialogs(Value: Boolean);
///simes    function GetPosition: TPoint;
    function GetTop: Integer;
    function GetLeft: Integer;
///simes    function GetOptions: TFindOptions;    see comment at UpdateDialogs
///simes    function GetHelpContext: THelpContext;   see comment at UpdateDialogs
///simes    function GetFindText: string;  see comment at UpdateDialogs
///simes    function GetReplaceText: string; see comment at UpdateDialogs
    function ReplaceOne(Sender: TObject): Boolean;
    procedure UpdateDialogs;
    procedure UpdateProperties(sender: TObject); /// simes - see comment at UpdateDialogs
    procedure NeedDialogs;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoOnFind(Sender: TObject); virtual;
    procedure DoOnReplace(Sender: TObject); virtual;
    procedure DoOnShow(Sender: TObject); virtual;
    procedure DoOnClose(Sender: TObject); virtual;
    procedure DoFailed(Sender: TObject); virtual;
    procedure DoReplacingAll; virtual;
    procedure DoReplacedAll(Sender: TObject); virtual;
    procedure DoProgress(Position: Integer; var Terminate: Boolean); virtual;
    procedure Loaded; override; /// simes
  public
    constructor Create(AOwner: TComponent); override;
    procedure Find; virtual;
    procedure FindAgain; virtual;
    procedure Replace; virtual;
    procedure ReplaceAll(const SearchText, ReplaceText: string); virtual;
    property Position: TPoint read FPosition write SetPosition;
    property Top: Integer read GetTop write SetDialogTop default -1;
    property Left: Integer read GetLeft write SetDialogLeft default -1;
  published
    property Fast: Boolean read FFast write FFast default False;
    property Options: TFindOptions read {///simes Get}FOptions write SetOptions;
    property EditControl: TCustomEdit read FEditControl write SetEditControl;
    property FindText: string read {///simes Get}FFindText write SetFindText;
    /// simes I believe KeepText should be removed - DoOnFind and DoOnReplace behave differently
    /// DoOnFind respects KeepText, but DoOnReplace with ReplaceAll option doesn't. What about KeepOptions?
    /// or should changed user options never be kept? I believe FindText, ReplaceText and Options should
    /// reflect the options as the end-user (not the component user) left them.
///Simes    property KeepText: Boolean read FKeepText write FKeepText default False;
    property ReplaceText: string read {///simes Get}FReplaceText write SetReplaceText;
    property ShowDialogs: Boolean read FShowDialogs write SetShowDialogs default True;
    property HelpContext: THelpContext read {///simes Get}FHelpContext write SetHelpContext default 0;
    property OnFind: TNotifyEvent read FOnFind write FOnFind;
    property OnReplace: TNotifyEvent read FOnReplace write FOnReplace;
    property OnReplacingAll: TNotifyEvent read FOnReplacingAll write FOnReplacingAll;
    property OnReplacedAll: TJvReplaceAllEvent read FOnReplacedAll write FOnReplacedAll;
    property OnNotFound: TNotifyEvent read FOnNotFound write FOnNotFound;
    property OnProgress: TJvReplaceProgressEvent read FOnProgress write FOnProgress;
  end;

implementation

uses
  {$IFDEF UNITVERSIONING}
  JclUnitVersioning,
  {$ENDIF UNITVERSIONING}
  Math,
  JvConsts, JvResources, JvTypes;

  /// Simes begin
  /// not needed, see FindInText
//type
//  TFoundText = record
//    StartAt: Longint;
//    EndAt: Longint;
//    isWhole: Boolean;
//    isSameCase: Boolean;
//  end;
  /// Simes end

procedure Error;
begin
  raise EJVCLException.CreateRes(@RsENoEditAssigned);
end;

{ utility }

function IsValidWholeWord(const S: string): Boolean;
begin
  Result := (Length(S) > 0) and not ((S[1] in IdentifierSymbols) or (S[Length(S)] in IdentifierSymbols));
end;

{ invert string }

function StrRev(const S: string): string;
var
  I, Len: Integer;
begin
  Len := Length(S);
  SetLength(Result, Len);
  for I := 1 to Len do
  begin
    Result[I] := S[Len];
    Dec(Len);
  end;
end;

{ Pascal adaption of a function originally in C }

function BoyerMoore(SubStr, S: PChar): Integer;
var
  CharJump, MatchJump, BackUp: array [0..255] of Integer;
  PatLen, TextLen, u, uA, uB, uText, uPat: Integer;
begin
  Result := 0;
  PatLen := StrLen(SubStr);
  TextLen := StrLen(S);

  FillChar(CharJump, 256 * SizeOf(Integer), 0);

  for u := 0 to PatLen do
    CharJump[Ord(SubStr[u])] := PatLen - u - 1;

  for u := 1 to PatLen - 1 do
    MatchJump[u] := 2 * PatLen - u;

  u := PatLen;
  uA := PatLen + 1;
  while u > 0 do
  begin
    BackUp[u] := uA;
    while (uA <= PatLen) and (SubStr[u - 1] <> SubStr[uA - 1]) do
    begin
      if MatchJump[uA] > PatLen - u then
        MatchJump[uA] := PatLen - u;
      uA := BackUp[uA];
    end;
    Dec(u);
    Dec(uA);
  end;

  for u := 1 to uA do
    if MatchJump[u] > PatLen + uA - u then
      MatchJump[u] := PatLen + uA - u;
  uB := BackUp[uA];

  while uA <= PatLen do
  begin
    while uA <= uB do
    begin
      if MatchJump[uA] > uB - uA + PatLen then
        MatchJump[uA] := uB - uA + PatLen;
      Inc(uA);
    end;
    uB := BackUp[uB];
  end;

  uPat := PatLen;
  uText := PatLen - 1;
  while (uText < TextLen) and (uPat <> 0) do
  begin
    if S[uText] = SubStr[uPat - 1] then
    begin
      Dec(uText);
      Dec(uPat);
    end
    else { mismatch - slide forward }
    begin
      uA := CharJump[Ord(S[uText])];
      uB := PatLen - uPat + 1;
      uText := uText + Max(uA, uB);
      uPat := PatLen;
    end;
  end;
  if uPat = 0 then
    Result := uText + 2;
end;

/// Simes begin
/// Simes. Routine modified to only return a match if it satisfies the WholeWord
/// and MatchCase requirements. (ToPos argument renamed to Len because it holds
/// the length - at least it does when it's passed to the Copy function)
/// return type modified since the isWhole and isCase members are no longer needed
/// and the EndAt was always the length of the search string.

/// issue 2513. Consider the case where the text contains "abc zzzz ABC". Case-
/// sensitivity is on, and the search string is ABC. FindInText would find the first
/// lowercase abc and return it's position, with isSameCase set to false. DoOnFind
/// would see that the returned result didn't satisfy the frMatchCase option, and say
/// "string not found".

{ Find text, return a longint }
function FindInText(const Text, Search: string; FromPos, Len: Integer; Fast,
  WholeWord, MatchCase: Boolean): longint;
var
  Found, SearchLen, TextLen: Integer;
  S: string;
begin
  Result := -1; // assume failure

  // first character in string is at position 1
  if FromPos = 0 then
    FromPos := 1;

  Found := 1;
  while (Result = -1) and (Found > 0) do
  begin
    if Fast then
      Found := BoyerMoore(PChar(AnsiUpperCase(Search)),
        PChar(AnsiUpperCase(Copy(Text, FromPos, Len))))
    else
      Found := Pos(AnsiUpperCase(Search), AnsiUpperCase(Copy(Text, FromPos,
        Len)));
    if Found > 0 then
    begin
      Result := Found + FromPos - 1;
      SearchLen := Length(Search);
      TextLen := Length(Text);
      FromPos := Result + SearchLen;
      // is match-case required and does it?
      if MatchCase and (AnsiCompareStr(Search, Copy(Text, Result, SearchLen)) <>
        0) then
        Result := -1
          // is whole-word-only required and is it?
      else if WholeWord and (SearchLen < TextLen) then
      begin
        // check for extremes...
        S := Copy(Text, Result - 1, SearchLen + 2);
        // check for match at beginning or end of string
        if Result = 1 then
          S := Copy(' ' + S, 1, SearchLen + 2);
        if Result - 1 + SearchLen + 2 > TextLen then
          S := Copy(S + ' ', TextLen - SearchLen - 1, SearchLen + 2);
        if not IsValidWholeWord(S) then
          result := -1;
      end;
    end;
  end;
end;
/// Simes end

{ invert and search }

/// Simes Begin - changed for extra arguments as FindInText
function FindInTextRev(const Text, Search: string; FromPos, Len: Integer;
  Fast, WholeWord, MatchCase: Boolean): longint;
begin
  Result := FindInText(StrRev(Text), StrRev(Search), FromPos, Len, Fast,
    WholeWord, MatchCase);
  if Result > -1 then
    Result := Length(Text) - (Result - 1) - (Length(Search) - 1);
end;
/// Simes end

constructor TJvFindReplace.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOwner := AOwner;
  FHelpContext := 0;
///simes  FKeepText := False;
  FShowDialogs := True;
  FPosition := Point(-1, -1);
end;

procedure TJvFindReplace.Find;
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateDialogs;
  if FShowDialogs then
    FFindDialog.Execute
  else
    DoOnFind(FFindDialog);
end;

procedure TJvFindReplace.FindAgain;
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateDialogs;
  DoOnFind(FFindDialog);
end;

procedure TJvFindReplace.Replace;
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateDialogs;

  if FShowDialogs then
    FReplaceDialog.Execute
  else
    DoOnReplace(FReplaceDialog);
end;

procedure TJvFindReplace.ReplaceAll(const SearchText, ReplaceText: string);
var
  Txt: string;
  FoundPos: longint;///simes TFoundText;
  SLen, RLen, TLen: Integer;
  Terminate: Boolean;
  WholeWord, MatchCase: Boolean; /// simes
begin
  if not Assigned(FEditControl) then
    Error;
  Terminate := False;
  UpdateDialogs;
  WholeWord := frWholeWord in FOptions; /// simes
  MatchCase := frMatchCase in FOptions; /// simes
  Txt := FEditControl.Text;
  SLen := Length(SearchText);
  RLen := Length(ReplaceText);
  TLen := Length(Txt);
  FoundPos := FindInText(Txt, SearchText, EditControl.SelStart + EditControl.SelLength,
    TLen, True, WholeWord, MatchCase); /// simes pass WholeWord and match case to FindInText
/// simes (incidentally, why is it passing True as the Fast argument. Should this uses the FFast member?)

  if FoundPos{///simes .StartAt} > -1 then
  begin
    DoReplacingAll;
    FNumberReplaced := 0;
    while FoundPos{///simes.StartAt} > -1 do
    begin
      Inc(FNumberReplaced);
      /// simes begin these lines no longer needed since found match must satisfy
      /// whole word and match case.
      /// (incidentally, it incremented the replacement count above, but may not
      /// have actually done the replacement if whole word or match case not satisfied. See issue 2508)
//      if (frWholeWord in TmpOptions) and not FoundPos.isWhole then
//      begin
//        FoundPos := FindInText(Txt, SearchText, FoundPos.StartAt + RLen + 1, TLen + (RLen - SLen), True);
//        Continue;
//      end;
//      if (frMatchCase in TmpOptions) and not FoundPos.isSameCase then
//      begin
//        FoundPos := FindInText(Txt, SearchText, FoundPos.StartAt + RLen + 1, TLen + (RLen - SLen), True);
//        Continue;
//      end;
      /// simes end

      Delete(Txt, FoundPos{///simes.StartAt + 1}, SLen);
      Insert(ReplaceText, Txt, FoundPos{///simes.StartAt + 1});
      FoundPos := FindInText(Txt, SearchText, FoundPos{///simes.StartAt} + RLen + 1, TLen + (RLen - SLen), True, WholeWord, MatchCase);/// simes

/// simes this is odd - why update progress only if returned match *position* is a multiple of 60? It could
/// make many replacements, and still never call the OnProgress event.
///      if FoundPos.StartAt mod 60 = 0 then
      if FNumberReplaced mod 10 = 0 then
      begin
        DoProgress(FoundPos{///simes.StartAt}, Terminate);
        if Terminate then
          Exit;
      end;
    end;
    FEditControl.Text := Txt;
    DoReplacedAll(FReplaceDialog);
  end
  else
    DoFailed(FReplaceDialog);
end;

function TJvFindReplace.ReplaceOne(Sender: TObject): Boolean;
var
  Equal: Integer;
  S, R: string;
begin
  Result := False;

  if FShowDialogs then
  begin
    S := TReplaceDialog(Sender).FindText;
    R := TReplaceDialog(Sender).ReplaceText;
  end
  else
  begin
    S := FFindText;
    R := FReplaceText;
  end;

  if frMatchCase in TFindDialog(Sender).Options then
    Equal := AnsiCompareStr(FEditControl.SelText, S)
  else
    Equal := AnsiCompareText(FEditControl.SelText, S);

  if Equal = 0 then
  begin
    Result := True;
    FEditControl.SelText := R;
    FEditControl.SetFocus;
  end;
end;

procedure TJvFindReplace.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FEditControl) then
    FEditControl := nil;
end;

procedure TJvFindReplace.NeedDialogs;
begin
  if not Assigned(FFindDialog) then
  begin
    FFindDialog := TFindDialog.Create(Self);
    FFindDialog.FindText := FFindText;
    FFindDialog.OnFind := DoOnFind;
    FFindDialog.Position := FPosition; /// simes initialise the dialogs position. See below
  end;
  if not Assigned(FReplaceDialog) then
  begin
    FReplaceDialog := TReplaceDialog.Create(Self);
    FReplaceDialog.FindText := FFindText;
    FReplaceDialog.ReplaceText := FReplaceText;
    FReplaceDialog.OnFind := DoOnFind;
    FReplaceDialog.OnReplace := DoOnReplace;
    FReplaceDialog.Position := FPosition;/// simes initialise the dialogs position. See below
  end;
end;

/// simes begin
/// doing "FFindDialog.Position := GetPosition;" here is no good. The dialogs exist
/// once NeedDialogs has completed, and so GetPosition then returns the dialogs actual
/// position, not the configured position.  It equates to
///    FFindDialog.Position := FFindDialog.Position;
/// I believe that the dialogs should always be initialised to the configured position
///    FFindDialog.Position := FPosition;
/// and FPosition should be updated to hold the dialogs current position as necessary.
/// (Hence the new UpdateProperties method, which does the reverse of the UpdateDialogs method.)
/// GetPosition then becomes redundant. Also setting the dialogs Top and Left properties
/// is pointless once the Position property has been set. They both refer to the same
/// thing.

/// Similarly with the help context, find text and replace text. Once the dialogs exist
/// the GetXXX returns the dialog's current properties, and not the value of this component's
/// current properties.

/// This also had the effect that you could show the dialog, then any changes to this components
/// Options, Position, FindText, ReplaceTest properties would be effectively ignored.
procedure TJvFindReplace.UpdateDialogs;
begin
///simes don't need to do this at design time
 if not (csDesigning in ComponentState) and not (csLoading in ComponentState) then
  begin
    NeedDialogs;

  FFindDialog.Position := FPosition;
///  {$IFDEF VCL}
///  FFindDialog.Top := GetTop;
///  FFindDialog.Left := GetLeft;
///  {$ENDIF VCL}
///  {$IFDEF VisualCLX}
///  FFindDialog.Position := Point(GetTop, GetLeft);
///  {$ENDIF VisualCLX}
  FFindDialog.Options := FOptions;
  FFindDialog.HelpContext := FHelpContext;
  FFindDialog.FindText := FFindText;

  FReplaceDialog.Position := FPosition;
///  {$IFDEF VCL}
///  FReplaceDialog.Top := GetTop;
///  FReplaceDialog.Left := GetLeft;
///  {$ENDIF VCL}
///  {$IFDEF VisualCLX}
///  FReplaceDialog.Position := Point(GetTop, GetLeft);
///  {$ENDIF VisualCLX}
  FReplaceDialog.Options := FOptions;
  FReplaceDialog.HelpContext := FHelpContext;
  FReplaceDialog.FindText := FFindText;
  FReplaceDialog.ReplaceText := FReplaceText;
  end;
end;

procedure TJvFindReplace.UpdateProperties(sender: TObject);
begin
  if Sender is TFindDialog then
  begin
    FPosition := TFindDialog(Sender).Position;
    FOptions := TFindDialog(Sender).Options;
    FHelpContext := TFindDialog(Sender).HelpContext;
    FFindText := TFindDialog(Sender).FindText;
  end;
  if Sender is TReplaceDialog then
    FReplaceText := TReplaceDialog(Sender).ReplaceText;
end;
/// simes end

procedure TJvFindReplace.DoOnFind(Sender: TObject);
var
  FoundPos: longint;/// simes TFoundText;
/// simes  S: string;
  Offset: Integer;
  WholeWord, MatchCase: Boolean; /// simes
begin
  /// simes begin
  /// update the local properties with the current values from the dialog
  /// in case the user has changed the options (or the find/replace text)
  UpdateProperties(Sender);
  WholeWord := frWholeWord in FOptions;
  MatchCase := frMatchCase in FOptions;

  /// simes after calling UpdateProperties, FFindText holds the string to be found
/// simes  if FShowDialogs then
/// simes   S := TFindDialog(Sender).FindText
/// simes  else
/// simes    S := FFindText;

/// simes I'm not sure about this!  This is inconsistent with DoOnReplace, where if
/// ReplaceAll is selected, FindText and ReplaceText are updated even if FKeepText
/// is false. I'd go for consistency, and remove the KeepText property entirely.
/// UpdateProperties now does this copy, and assumes FKeepText option is removed.
///  if FKeepText then
///  begin
///    FFindText := TFindDialog(Sender).FindText;
///    FReplaceText := TReplaceDialog(Sender).ReplaceText;
///  end;

/// simes the off set depends on the direction. If searching up, don't start
/// from the end of the selection, or else it will always return the same match
  if not (frDown in FOptions) then
  begin
    Offset := FEditControl.SelStart;
    if Offset = 0 then
      Offset := 1;
    FoundPos := FindInTextRev(FEditControl.Text, FFindText,
      Length(FEditControl.Text) - Offset, Length(FEditControl.Text), FFast,
      WholeWord, MatchCase)
  end else
  begin
    Offset := FEditControl.SelStart + FEditControl.SelLength;
    if Offset = 0 then
      Offset := 1;
    FoundPos := FindInText(FEditControl.Text, FFindText, Offset,
      Length(FEditControl.Text), FFast, WholeWord, MatchCase);
  end;

  if FoundPos{///simes.StartAt} > -1 then
  begin
   /// simes begin - we should not fail in this circumstance. The desired text
   /// could still be present further down the string. But now returned match
   /// will already be checked for whole word and match case
///    if (frWholeWord in TFindDialog(Sender).Options) and not FoundPos.isWhole then
///      DoFailed(Sender)
///    else
///    if (frMatchCase in TFindDialog(Sender).Options) and not FoundPos.isSameCase then
///      DoFailed(Sender)
///    else
///    begin
      FEditControl.SetFocus;
      FEditControl.SelStart := FoundPos{//simes.StartAt} - 1;// simes
      FEditControl.SelLength := Length(FFindText); ///simes
      {$IFDEF VCL}
      SendMessage(FEditControl.Handle, EM_SCROLLCARET, 0, 0);
      {$ENDIF VCL}
      if Assigned(FOnFind) then
        FOnFind(Self);
/// simes end
  end
  else
    DoFailed(Sender);
end;

procedure TJvFindReplace.DoOnReplace(Sender: TObject);
begin
  UpdateProperties(Sender); // simes get updated stuff from dialog (Text was done below for ReplaceAll,
                            ///except for Options - issue 2509)

/// simes This finds the first occurence. If ReplaceAll is selected, this first occurence won't be replaced
/// so I moved this block into the non-ReplaceAll branch below
///  if FEditControl.SelLength < 1 then
///    DoOnFind(Sender);
///  if FEditControl.SelLength < 1 then
///    Exit;

  if frReplaceAll in FOptions { ///simes TFindDialog(Sender).Options} then
  begin
//simes    SetFindText(FReplaceDialog.FindText);
//simes    SetReplaceText(FReplaceDialog.ReplaceText);
    ReplaceAll(FFindText, FReplaceText);
    if Assigned(FOnReplace) then
      FOnReplace(Self);
  end
  else
  begin
  if FEditControl.SelLength < 1 then
    DoOnFind(Sender);
  if FEditControl.SelLength < 1 then
    Exit;

    ReplaceOne(Sender);
    if Assigned(FOnReplace) then
      FOnReplace(Self);
    DoOnFind(Sender);
  end;
end;

procedure TJvFindReplace.DoOnShow(Sender: TObject);
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateDialogs;
  if Assigned(FOnShow) then
    FOnShow(Self);
end;

procedure TJvFindReplace.DoOnClose(Sender: TObject);
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateProperties(Sender); ///simes update local properties from dialog
  UpdateDialogs;
  if Assigned(FOnClose) then
    FOnClose(Self);
end;

procedure TJvFindReplace.DoFailed(Sender: TObject);
var
  FCaption: string;
begin
  if not Assigned(FEditControl) then
    Error;
  UpdateProperties(Sender); ///simes update local properties from dialog
                            ///simes (or else OnNotFound cannot see what text was not found. Issue 2505)
  if Assigned(FOnNotFound) then
    FOnNotFound(Self);
  if not FShowDialogs then
    Exit;

  if Sender = FReplaceDialog then
    FCaption := RsReplaceCaption
  else
    FCaption := RsFindCaption;

  MessageBox(
    {$IFDEF VCL}
    TFindDialog(Sender).Handle,
    {$ENDIF VCL}
    {$IFDEF VisualCLX}
    TFindDialog(Sender).Form.Handle,
    {$ENDIF VisualCLX}
    PChar(Format(RsNotFound, [{///simes TFindDialog(Sender).}FFindText])),
    PChar(FCaption), MB_OK or MB_ICONINFORMATION);
end;

procedure TJvFindReplace.DoReplacingAll;
begin
  if Assigned(FOnReplacingAll) then
    FOnReplacingAll(Self);
end;

procedure TJvFindReplace.DoReplacedAll(Sender: TObject);
begin
  UpdateProperties(Sender); ///simes update local properties from dialog
  if FShowDialogs then
  begin
    MessageBox(
      {$IFDEF VCL}
      TFindDialog(Sender).Handle,
      {$ENDIF VCL}
      {$IFDEF VisualCLX}
      TFindDialog(Sender).Form.Handle,
      {$ENDIF VisualCLX}
      PChar(Format(RsXOccurencesReplaced, [FNumberReplaced, {///simesTFindDialog(Sender).}FFindText])),
      PChar(RsReplaceCaption), MB_OK or MB_ICONINFORMATION);
  end;

  if Assigned(FOnReplacedAll) then
    FOnReplacedAll(Self, FNumberReplaced);
end;

procedure TJvFindReplace.DoProgress(Position: Integer; var Terminate: Boolean);
begin
  if Assigned(FOnProgress) then
    FOnProgress(Self, Position, Terminate);
end;

procedure TJvFindReplace.SetPosition(Value: TPoint);
begin
  FPosition := Value;
  UpdateDialogs;
end;

procedure TJvFindReplace.SetDialogTop(Value: Integer);
begin
//simes  FTop := Value;
  FPosition.Y := Value; //simes
  UpdateDialogs;
end;

procedure TJvFindReplace.SetDialogLeft(Value: Integer);
begin
///simes  FLeft := Value;
  FPosition.X := Value; //simes
  UpdateDialogs;
end;

procedure TJvFindReplace.SetOptions(Value: TFindOptions);
begin
  FOptions := Value;
  UpdateDialogs;
end;

procedure TJvFindReplace.SetEditControl(Value: TCustomEdit);
begin
  FEditControl := Value;
  if Value <> nil then
    Value.FreeNotification(Self);
end;

procedure TJvFindReplace.SetFindText(const Value: string);
begin
  FFindText := Value;
  UpdateDialogs;///simes
///simes  if Assigned(FFindDialog) then
///simes    FFindDialog.FindText := Value;
///simes  if Assigned(FReplaceDialog) then
///simes    FReplaceDialog.FindText := Value;
end;

procedure TJvFindReplace.SetShowDialogs(Value: Boolean);
begin
  if FShowDialogs <> Value then
    FShowDialogs := Value;
  if not Value then
  begin
    NeedDialogs;
    {$IFDEF VCL}
    FFindDialog.CloseDialog;
    FReplaceDialog.CloseDialog;
    {$ENDIF VCL}
    {$IFDEF VisualCLX}
    FFindDialog.Form.Close;
    FReplaceDialog.Form.Close;
    {$ENDIF VisualCLX}
  end;
end;

procedure TJvFindReplace.SetReplaceText(const Value: string);
begin
  FReplaceText := Value;
  UpdateDialogs;///simes
///simes  if Assigned(FReplaceDialog) then
///simes    FReplaceDialog.ReplaceText := Value;
end;

procedure TJvFindReplace.SetHelpContext(Value: THelpContext);
begin
  FHelpContext := Value;
  UpdateDialogs;
end;

{///simes  FPosition now always holds the current dialogs position
function TJvFindReplace.GetPosition: TPoint;
begin
  if (csDesigning in ComponentState) or not Assigned(FFindDialog) then
    Result := FPosition
  else
    Result := FFindDialog.Position;
end;
}

function TJvFindReplace.GetTop: Integer;
begin
///simes  if (csDesigning in ComponentState) or not Assigned(FFindDialog) then
///simes  FPosition now always holds the current dialogs position
    Result := FPosition.Y
///simes  else
///simes    {$IFDEF VCL}
///simes    Result := FFindDialog.Top;
///simes    {$ENDIF VCL}
///simes    {$IFDEF VisualCLX}
///simes    Result := FFindDialog.Position.Y;
///simes    {$ENDIF VisualCLX}
end;

function TJvFindReplace.GetLeft: Integer;
begin
///simes  if (csDesigning in ComponentState) or not Assigned(FFindDialog) then
///simes  FPosition now always holds the current dialogs position
    Result := FPosition.X
///simes  else
///simes    {$IFDEF VCL}
///simes    Result := FFindDialog.Left;
///simes    {$ENDIF VCL}
///simes    {$IFDEF VisualCLX}
///simes    Result := FFindDialog.Position.X;
///simes    {$ENDIF VisualCLX}
end;

{/// simes begin
function TJvFindReplace.GetOptions: TFindOptions;
begin
  if (csDesigning in ComponentState) or not Assigned(FFindDialog) then
    Result := FOptions
  else
    Result := FFindDialog.Options;
end;

function TJvFindReplace.GetHelpContext: THelpContext;
begin
  if (csDesigning in ComponentState) or not Assigned(FFindDialog) then
    Result := FHelpContext
  else
    Result := FFindDialog.HelpContext;
end;

function TJvFindReplace.GetFindText: string;
begin
  if not FShowDialogs or (csDesigning in ComponentState) or not Assigned(FFindDialog) then
    Result := FFindText
  else
    Result := FFindDialog.FindText;
end;

function TJvFindReplace.GetReplaceText: string;
begin
  if not FShowDialogs or (csDesigning in ComponentState) or not Assigned(FReplaceDialog) then
    Result := FReplaceText
  else
    Result := FReplaceDialog.ReplaceText;
end;
///simes end }

/// simes begin
procedure TJvFindReplace.Loaded;
begin
  inherited;
  UpdateDialogs;
end;
///simes end

{$IFDEF UNITVERSIONING}
const
  UnitVersioning: TUnitVersionInfo = (
    RCSfile: '$RCSfile: JvFindReplace.pas,v $';
    Revision: '$Revision: 1.26 $';
    Date: '$Date: 2005/01/17 20:57:40 $';
    LogPath: 'JVCL\run'
  );

initialization
  RegisterUnitVersion(HInstance, UnitVersioning);

finalization
  UnregisterUnitVersion(HInstance);
{$ENDIF UNITVERSIONING}

end.

